if LxpTree then return end

-- LxpTree: build a DOM-style tree from an XML document.

-- In general, call as tree = LxpTree.parse(content)

-- As a special hack for XML-RPC, a mode to ignore character data
-- except for named elements is available.  Call
-- p=LxpTree.makeParser(), and set p.filter_chardata true.  Tags not
-- named in the p.keep_chardata table will be ignored.  You probably
-- want to set p.merge_chardata as well; this merges all character
-- data in a tag.

local Public, Private = {}, {}
LxpTree = Public

function Private.top(l)
  return l[getn(l)]
end

local parserTemplate = {}

function parserTemplate:starttag(name, attr)
  local t = {name}
  if attr then t.attr=attr end
  self.str = nil
  tinsert(%Private.top(self.stack), t)
  tinsert(self.stack, t)
  if self.filter_chardata and self.keep_chardata[name] then
    self.want_chardata = 1
  end
end

function parserTemplate:endtag(name)
  if self.merge_chardata and self.str and self.keep_chardata[name] then
    tinsert(%Private.top(self.stack), self.str)
  end
  self.str = nil
  self.want_chardata = nil
  tremove(self.stack)
end

function parserTemplate:chardata(s)
  -- tinsert(self.stack[getn(self.stack)], s)
  if self.filter_chardata and not self.want_chardata then return end
  if self.merge_chardata then
    if self.str then 
      self.str = self.str .. s
    else
      self.str = s
    end
  else
    tinsert(%Private.top(self.stack), s)
  end
end

function parserTemplate:parse(s)
  if not lxp_parse(self, s) or not lxp_parse_end(self) then
  error("expat parse error "..lxp_error_code(self).."error string"..lxp_error_string(self).." at line"..lxp_get_line_number(self).."column"..lxp_get_column_number(self))
end  
return self.stack[1][1]
end

Private.parserTemplate = parserTemplate
parserTemplate = nil

function Public.makeParser()
  local p=lxp_make_parser()
  for i,v in %Private.parserTemplate do
    p[i] = v
  end
  p.stack={{}}
  return p
end

function Public.parse(s)
  local p = %Public.makeParser()
  return p:parse(s)
end
