if XMLGen then return end

local Public, Private = {}, {}
XMLGen = Public

-- Right out of LTN 9

function Private.newBuffer ()
  return {n=0}     -- 'n' counts number of elements in the stack
end

function Private.addString (stack, s)
  tinsert(stack, s)       -- push 's' into the top of the stack
  for i=stack.n-1, 1, -1 do
    if strlen(stack[i]) > strlen(stack[i+1]) then break end
    stack[i] = stack[i]..tremove(stack)
  end
end

function Private.toString (stack)
  for i=stack.n-1, 1, -1 do
    stack[i] = stack[i]..tremove(stack)
  end
  return stack[1]
end

function Public.quote(s)
  s = gsub(s, "&", "&amp;")
  s = gsub(s, "<", "&lt;")
  s = gsub(s, ">", "&gt;")
  return s
end

function Public.quoteForSingle(s)
  s = %Public.quote(s)
  s = gsub(s, "'", "&apos;")
  return s
end

function Private.marshallToBuffer(s, l)
  local addString = %Private.addString
  local name = l[1]
  if l.attr then
    addString(s, "<"..name)
    for i,v in l.attr do
      addString(s, " "..i.."='"..%Public.quoteForSingle(v).."'")
    end
    addString(s, ">")
  else
    addString(s, "<"..name..">")
  end
  for i = 2,getn(l) do
    local elt = l[i]
    if type(elt) == "table" then
      %Private.marshallToBuffer(s, elt)
    elseif type(elt) == "string" then
      addString(s, %Public.quote(elt))
    else
      error("non-table, non-string in marshalling")
    end
  end
  addString(s, "</"..name..">")
end

function Public.treeToString(t)
  local s = %Private.newBuffer()
  %Private.marshallToBuffer(s, t)
  return %Private.toString(s)
end

